@@ -1,12 +1,13 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 #########################################################
-## Thai Language Toolkit : version  1.4.1
+## Thai Language Toolkit : version  1.5.7
 ## Chulalongkorn University
 ## written by Wirote Aroonmanakun
 ## Implemented :
 ##      upgrade to gensim 4.0
 ##      Corpus_build(DIR), W2V_train(Corpus), D2V_train(Corpus)
+##      download_TNCw2v(), download_TNC3g()
 ##      TNC_load(), TNC3g_load(), trigram_load(Filename), unigram(w1), bigram(w1,w2), trigram(w1,w2,w3)
 ##      collocates(w,SPAN,STAT,DIR,LIMIT, MINFQ) = [wa,wb,wc]
 ##      w2v_exist(w), similar_words(w), similarlity(w1,w2), outofgroup(WORDLST), analogy(w1,w2,w3) 
@@ -17,6 +18,8 @@ import os
 import math
 from collections import defaultdict
 from operator import itemgetter
+import requests
+
 import gensim
 from gensim.models import Word2Vec, KeyedVectors
 from gensim.models.doc2vec import Doc2Vec, TaggedDocument
@@ -32,9 +35,30 @@ else:
     mplAvailable = True
 import numpy
 
+import tltk
+from tltk import nlp
+
 #import gensim, logging
 #logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s', level=logging.INFO)
 
+def download_TNCw2v():
+    url = 'https://www.arts.chula.ac.th/ling/wp-content/uploads/TNC5cmodel2.bin'
+    r = requests.get(url, allow_redirects=True)
+    open(ATA_PATH +'/TNC5cmodel2.bin','wb').write(r.content)
+    return()
+
+def download_TNC3g(): 
+    url = 'https://www.arts.chula.ac.th/ling/wp-content/uploads/TNC.3g'
+    r = requests.get(url, allow_redirects=True)
+    open(ATA_PATH +'/TNC.3g','wb').write(r.content)
+    return()
+
+def download_TNCd2v():
+    url = 'https://www.arts.chula.ac.th/ling/wp-content/uploads/TNCd2v.bin'
+    r = requests.get(url, allow_redirects=True)
+    open(ATA_PATH +'/TNCd2v.bin','wb').write(r.content)
+    return()
+
 def TNC3g_load():
     global TriCount
     global BiCount
@@ -302,7 +326,7 @@ def compute_colloc(stat,w1,w2):
 
 #######################################################################
 ### Load Corpus from text files in a directory
-def Corpus_build(dir, sep="par", lang="th",stopword="y", filetype="txt"):
+def Corpus_build(dir, sep="file", lang="th",stopword="y", filetype="txt"):
     ThaiStopwords = ['ที่','การ','เป็น','ใน','ของ','มี','และ','ได้','ให้','ว่า','ไป','มา','ก็','ๆ','ความ','กับ','หรือ','อยู่','กัน','จาก','นี้','แต่','อย่าง','ด้วย','เขา','ขึ้น','นั้น','ผู้','ซึ่ง','ตาม','โดย','ยัง','เพื่อ','อีก','เมื่อ','ถึง','เพราะ','ออก','คือ','จึง','กว่า','ไว้','ถ้า','อะไร','ลง','แบบ','ทุก','อื่น','เช่น','น่า','สามารถ','แห่ง','นี่','ใคร','ใด','เอง','จะ','คง','เคย','อาจ','ต้อง','แล้ว','ครับ','คะ','ค่ะ','นะ','ไม่','ไม่ได้']
     text_corpus = []
 #    count = defaultdict(int)
@@ -314,10 +338,24 @@ def Corpus_build(dir, sep="par", lang="th",stopword="y", filetype="txt"):
             if not fname.endswith('.'+filetype):
                 continue
             with open(dirName + '/' + fname,encoding='utf8') as f:
-                print(fname)
-                if sep == 'file':
-                    lines = f.readlines()
-                    doc = ' '.join(lines)
+                if sep == 'edu':
+                    txt = f.read()
+                    doc = nlp.segment(txt)
+                    doc = doc.replace('<s/>','')
+                    doc = doc.replace('\n','')
+                    for u in doc.split('<u/>'):
+                        wrdlst = re.split("[, |?:!\^]+",u)
+                        if '' in wrdlst : wrdlst.remove('')
+                        if wrdlst != []:
+                            if lang =='th' and stopword == 'y':
+                                wrdlst_wo_sw = [w for w in wrdlst if not w in ThaiStopwords]
+                                text_corpus.append(wrdlst_wo_sw) 
+                            else:
+                                text_corpus.append(wrdlst) 
+                elif sep == 'file':
+                    txt = f.read()
+                    doc = nlp.word_segment(txt)
+#                    doc = ' '.join(lines)
                     doc = doc.replace('<s/>','')
                     doc = doc.replace('\n','')
                     wrdlst = re.split("[, |?:!\^]+",doc)
@@ -329,7 +367,8 @@ def Corpus_build(dir, sep="par", lang="th",stopword="y", filetype="txt"):
                         else:
                             text_corpus.append(wrdlst) 
                 else:
-                    for doc in f.readlines():
+                    for line in f.readlines():
+                        doc = nlp.word_segment(line)
                         doc = doc.replace('\n','')
                         doc = doc.replace('<s/>','')
                         wrdlst = re.split("[, |?:!\^]+",doc)
@@ -401,8 +440,46 @@ class Xwordlist:
         Y = B.keys()
         return(list(set(X)|set(Y))) 
 
+
+
+'''
+############ load D2V model from TNC
+def D2V_load(File="TNCd2v.bin"):
+    global d2v_model
+
+    path = os.path.abspath(__file__)
+    ATA_PATH = os.path.dirname(path)
+    try:
+        d2v_model = Doc2Vec.load(File,'rb')
+    except IOError:
+        d2v_model = Doc2Vec.load(ATA_PATH +'/' +"TNCd2v.bin",'rb')
+    return(1)  
+
+def similar_edu(seg,n=1):
+    global d2v_model
+
+    x = seg.split('|')
+    #create vector for that segment
+    newvect = d2v_model.infer_vector(x)
+    #find the most similar top 10 vectors
+    sims = d2v_model.dv.most_similar([newvect])
+    if n == 1:
+        return(pars[int(sims[0][0])])
+    else:
+        ct = 0
+        result = []
+        while ct < n:
+            result.append(pars[int(sims[0][ct])])
+            ct += 1
+        return(result)    
+    #print using index [0][0], [0][1]
+#    print('1.',pars[int(sims[0][0])])
+#    print('2.',pars[int(sims[0][1])])
+
+'''
+
 #######################################################################
-### word2vec model created from TNC 3.0 gensim 4.0
+#### word2vec model created from TNC 3.0 gensim 4.0
 
 def W2V_load(File="TNCc5model2.bin"):
     global w2v_model
@@ -568,7 +645,7 @@ def w2v_plot(wrdlst):
     return(1)
 
     
-
+###  man1 : king2  woman3 : queen
 def analogy(w1,w2,w3, n=1):   #king - man + woman = queen    
     global w2v_model
     try:
@@ -577,7 +654,7 @@ def analogy(w1,w2,w3, n=1):   #king - man + woman = queen
       w2v_load()
     vocabs = list(w2v_model.wv.index_to_key)
     if w1 in vocabs and w2 in vocabs and w3 in vocabs:    
-        return(w2v_model.wv.most_similar(positive=[w3, w1], negative=[w2], topn=n))
+        return(w2v_model.wv.most_similar(positive=[w3, w2], negative=[w1], topn=n))
     else:
         return([])
 
@@ -614,24 +691,17 @@ def w2v_compare_color(wds):
 
 
 
-'''
+
+
 ## testing area
-
-# read all .wsg files in the directory
-pars = Corpus_build('/Users/macbook/Cloud/Dropbox/Corpus/Thaipublica/wsg',filetype="wsg")
+'''
+pars = Corpus_build('/Users/macbook/Cloud/Dropbox/Corpus/corpora/ThaiNews',sep="edu",filetype="txt")
 print(pars[0:3])
-x1 = Corpus()
-print(x1.totalword(pars))
-c1 = Corpus()
-c2 = Corpus()
-Xcomp = Comp_wordlist()
-print(Xcomp.onlyA(c1.frequency(pars[:100]),c2.frequency(pars[101:])))
-
-W2V_load('TNCc5model2.bin')
-print(similar_words('ขบ'))
 
 #train Doc2Vec model
 model = D2V_train(pars)
+
+
 #a new paragraph
 x = 'การ|ทดลอง|ภาค|สนาม|โดย|ทีม|ผู้|เชี่ยวชาญ|เรื่อง|การ|ให้|และ|การ|ช่วยเหลือ|ผู้|อื่น'.split('|')
 #create vector for that paragraph
@@ -642,6 +712,21 @@ print(sims)
 #print using index [0][0], [0][1]
 print('1.',pars[int(sims[0][0])])
 print('2.',pars[int(sims[0][1])])
+
+
+# read all .wsg files in the directory
+pars = Corpus_build('/Users/macbook/Cloud/Dropbox/Corpus/Thaipublica/wsg',filetype="wsg")
+print(pars[0:3])
+x1 = Corpus()
+print(x1.totalword(pars))
+c1 = Corpus()
+c2 = Corpus()
+Xcomp = Xwordlist()
+print(Xcomp.onlyA(c1.frequency(pars[:100]),c2.frequency(pars[101:])))
+
+W2V_load('TNCc5model2.bin')
+print(similar_words('ขบ'))
+
 #w2v_load()
 #w2v_plot("ผู้ชาย ผู้หญิง เก่ง ฉลาด สวย หล่อ".split(" "))
 #w2v_compare_color("ผู้ชาย ผู้หญิง เก่ง ฉลาด สวย หล่อ".split(" "))
